//+------------------------------------------------------------------+
//|                                            Twiggs Money Flow.mq4 |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " Mr. Colin Twiggs"
#property link      "http://www.incrediblecharts.com/technical/twiggs_money_flow.php"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrLimeGreen
#property indicator_color2  clrOrangeRed
#property indicator_color3  clrDarkGray
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern int  Prd             = 21;
extern bool alertsOn        = true;
extern bool alertsOnCurrent = true;
extern bool alertsMessage   = true;
extern bool alertsSound     = true;
extern bool alertsNotify    = false;
extern bool alertsEmail     = false;
extern string soundFile     = "alert2.wav";

//
//
//
//
//

double ExtMapBuffer1[];
double WildersBuffer[];
double WVBuffer[];
double ADVBuffer[];
double SmV[];
double SmA[],histou[],histod[],trend[];


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//

int init()
{

   IndicatorBuffers(9);
   SetIndexBuffer(0,histou); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,histod); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,ExtMapBuffer1);
   SetIndexBuffer(3,WildersBuffer);
   SetIndexBuffer(4,WVBuffer);
   SetIndexBuffer(5,ADVBuffer);
   SetIndexBuffer(6,SmV);
   SetIndexBuffer(7,SmA);
   SetIndexBuffer(8,trend);
   SetLevelValue(0,0.0);
 //  SetLevelValue(1,0.1);
 //  SetLevelValue(2,0.25);
 //  SetLevelValue(3,-0.1);
 //  SetLevelValue(4,-0.25);
   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int deinit()
{

   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

     //
     //
     //
     //
     //

     for (int i = limit; i >= 0; i--)
     {

       double TRH = iHigh(NULL, 0, iHighest(NULL, 0, MODE_HIGH, 1, i)) + 1*Point;
       double TRL = iLow(NULL, 0, iLowest(NULL, 0, MODE_LOW, 1, i)) - 1*Point;
       double TR  =  TRH-TRL;
       if (TR==0) { ADVBuffer[i] = ((Close[i]-TRL)-(TRH-Close[i]))/ (999999)*Volume[i];  }
       else       { ADVBuffer[i] = ((Close[i]-TRL)-(TRH-Close[i]))/ (TR)*Volume[i];      }
      
        WVBuffer[i]     = Volume[i] + (iVolume(NULL,0,i)*0);  
        WildersBuffer[i]= iMA(NULL,0,(Prd*2)-1,0,MODE_EMA,PRICE_CLOSE,i);
   
      
    }

                  
      for (i = limit; i >= 0; i--)
      {
        SmV[i] = iMAOnArray(WVBuffer,0,(Prd*2)-1,0,MODE_EMA,i);
        SmA[i]= iMAOnArray(ADVBuffer,0,(Prd*2)-1,0,MODE_EMA,i);
      
       if (TRH != 0)
       {
         ExtMapBuffer1[i] = SmA[i]/SmV[i];
       }
       histou[i] = EMPTY_VALUE;
       histod[i] = EMPTY_VALUE;
       trend[i]  = trend[i+1];
       if (ExtMapBuffer1[i]>0) { trend[i] =  1; histou[i] = ExtMapBuffer1[i]; }
       if (ExtMapBuffer1[i]<0) { trend[i] = -1; histod[i] = ExtMapBuffer1[i]; }
   }
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar+1] ==  1 && trend[whichBar] != 1) doAlert(whichBar,"up");
         if (trend[whichBar+1] == -1 && trend[whichBar] !=-1) doAlert(whichBar,"down");
      } 
   }               
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Twiggs money flow trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsNotify)  SendNotification(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Twiggs money flow "),message);
          if (alertsSound)   PlaySound(soundFile);
   }
}



